#!/usr/bin/env python3

import glfw

from OpenGL.GL import *

def keyboard(window, key, scancode, action, mods):

  if key == glfw.KEY_Q and action == glfw.PRESS:
    print("Q pressed")
    glfw.set_window_should_close(window, True)

  if key == glfw.KEY_ESCAPE and action == glfw.PRESS:
    print("ESC pressed")
    glfw.set_window_should_close(window, True)

def mouse_motion(window, mx, my):

    ww, wh = glfw.get_window_size(window)
    print("window size: ", ww, wh)

    # normalized coords
    x = mx/ww
    y = my/wh

    print("x, y: ", x, y)

def render_matrix(window, rows, cols):

    # render
    ww, wh = glfw.get_framebuffer_size(window)
#   ww, wh = glfw.get_window_size(window)
#   print("window size: ", ww, wh)

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

    # draw grid lines
    # glViewport(x,y,width,height)
    glViewport(0, 0, ww, wh)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()

    # l,r,b,t,znear,zfar
    glOrtho(0.0, ww, 0.0, wh, 0.0, 1.0)

    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

    # yellow
    glColor4f(255.0/255.0, 247.0/255.0, 188.0/255.0, 1.0)
    glBegin(GL_LINES)
    for i in range(rows):
      glVertex3f(0,(i/rows)*wh,0.0)
      glVertex3f(ww,(i/rows)*wh,0.0)
    glEnd()
    glBegin(GL_LINES)
    for j in range(cols):
      glVertex3f((j/cols)*ww,0,0.0)
      glVertex3f((j/cols)*ww,wh,0.0)
    glEnd()

def render_grid(window):

    ww, wh = glfw.get_framebuffer_size(window)
#   ww, wh = glfw.get_window_size(window)
#   print("window size: ", ww, wh)

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

    # draw grid lines
    # glViewport(x,y,width,height)
    glViewport(0, 0, ww, wh)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()

    # l,r,b,t,znear,zfar
    glOrtho(0.0, ww, 0.0, wh, 0.0, 1.0)

    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

    n = 12
    n2 = n*n

    # yellow
    glColor4f(255.0/255.0, 247.0/255.0, 188.0/255.0, 1.0)
    for i in range(int(n)):
        for j in range(int(n)):
    
            cell = int(i*n + j)
#           print("n: %d, (i*n+j): %d" % ( n, cell))
    
            glColor4f(cell/n2, cell/n2, cell/n2, 1.0)
            glBegin(GL_QUADS)
            # tl, bl, br, tr
            glVertex2f((j  )/n*ww, wh-(i  )/n*wh)
            glVertex2f((j+1)/n*ww, wh-(i  )/n*wh)
            glVertex2f((j+1)/n*ww, wh-(i+1)/n*wh)
            glVertex2f((j  )/n*ww, wh-(i+1)/n*wh)
            glEnd()

def main():
    # initialize glfw
    if not glfw.init():
        return

    # create a windowed mode window and its OpenGL context
    glfw.window_hint(glfw.CLIENT_API, glfw.OPENGL_API)
#   glfw.window_hint(glfw.CONTEXT_VERSION_MAJOR, 3)
#   glfw.window_hint(glfw.CONTEXT_VERSION_MINOR, 2)
#   glfw.window_hint(glfw.OPENGL_PROFILE, glfw.OPENGL_CORE_PROFILE)

    glfw.window_hint(glfw.RESIZABLE, GL_TRUE)

    window1 = glfw.create_window(640,480,"window1",None,None)
    if not window1:
        glfw.terminate()
        return
    glfw.set_key_callback(window1, keyboard)

    window2 = glfw.create_window(640,480,"window2",None,None)
    if not window2:
        glfw.terminate()
        return
    glfw.set_key_callback(window2, keyboard)
    glfw.set_cursor_pos_callback(window2, mouse_motion)

    # loop until user closes window
    while(not glfw.window_should_close(window1) and
          not glfw.window_should_close(window2)):

        # make window's context current
        glfw.make_context_current(window1)

        # render here
        render_grid(window1)

        # swap buffers
        glfw.swap_buffers(window1)

        # make window's context current
        glfw.make_context_current(window2)

        # render here
        render_matrix(window2, 3, 4)

        # swap buffers
        glfw.swap_buffers(window2)

        # poll for and process events
        glfw.poll_events()

    glfw.terminate()

if __name__ == '__main__':
    main()
