#!/usr/bin/env python3

import platform, sys, os, math
import argparse
import numpy as np
import time
import datetime
from collections import deque
from PIL import Image
import cv2

from state import State
from point import Point
from fixation import Fixation
from bw import Butterworth as BW
from sg import SG
from monitor import Monitor
from calibration import Calibration
from gazept import Gazept
from rstats import RunningStats
from grid import Grid
from tm import TM

from math import *
from OpenGL.GL import *
from OpenGL.GLUT import *
from OpenGL.GLU import *

print(OpenGL.__file__)

# prevent Python from writing .pyc files
sys.dont_write_bytecode = True

# globals
global window1, window2

def clamp(x,y):
    if x < 0:
      x = 0.0
    if y < 0:
      y = 0.0
    if x > 1:
      x = 1.0
    if y > 1:
      y = 1.0
    return x,y

def idle():

# glutPostRedisplay()
  pass

def timer(extra):
  glutPostRedisplay()

  # re-arm timer
  glutTimerFunc(60,timer,0)

def reshape(width, height):
  # only call glViewport once here, not in display()!
  glViewport(0, 0, width, height)

  glMatrixMode(GL_PROJECTION)
  glLoadIdentity()

  glMatrixMode(GL_MODELVIEW)
  glLoadIdentity()

# monitor.set_window_width(glutGet(GLUT_WINDOW_WIDTH))
# monitor.set_window_height(glutGet(GLUT_WINDOW_HEIGHT))

def display():

  global window1, window2

  if window1 is not None:

    glutSetWindow(window1)

    ww = glutGet(GLUT_WINDOW_WIDTH)
    wh = glutGet(GLUT_WINDOW_HEIGHT)

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
    
    # draw grid lines
    # glViewport(x,y,width,height)
    glViewport(0, 0, ww, wh)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()

    # l,r,b,t,znear,zfar
    glOrtho(0.0, ww, 0.0, wh, 0.0, 1.0)
    
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

    glutSwapBuffers()

  if window2 is not None:
    glutSetWindow(window2)

    ww = glutGet(GLUT_WINDOW_WIDTH)
    wh = glutGet(GLUT_WINDOW_HEIGHT)

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

    # draw grid lines
    # glViewport(x,y,width,height)
    glViewport(0, 0, ww, wh)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()

    # l,r,b,t,znear,zfar
    glOrtho(0.0, ww, 0.0, wh, 0.0, 1.0)

    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

    n = 12.
    n2 = n*n

    # yellow
    glColor4f(255.0/255.0, 247.0/255.0, 188.0/255.0, 1.0)
    for i in range(int(n)):
      for j in range(int(n)):
    
        cell = int(i*n + j)
#       print("n: %d, (i*n+j): %d" % ( n, cell))
    
        glColor4f(cell/n2, cell/n2, cell/n2, 1.0)
        glBegin(GL_QUADS)
        # tl, bl, br, tr
        glVertex2f((j  )/n*ww, wh-(i  )/n*wh)
        glVertex2f((j+1)/n*ww, wh-(i  )/n*wh)
        glVertex2f((j+1)/n*ww, wh-(i+1)/n*wh)
        glVertex2f((j  )/n*ww, wh-(i+1)/n*wh)
        glEnd()
    
    glutSwapBuffers()

def draw_rect(x, y, width, height):
  # draw in screen coordinates
  glBegin(GL_QUADS)
  glVertex2f(x - (width / 2), y - (height / 2))
  glVertex2f(x + (width / 2), y - (height / 2))
  glVertex2f(x + (width / 2), y + (height / 2))
  glVertex2f(x - (width / 2), y + (height / 2))
  glEnd()

def draw_circ(cx, cy, r, num_segments, type):
  theta = 2 * 3.1415926 / float(num_segments)
  # tangential factor
  tanf = tan(theta)
  # radial factor
  radf = cos(theta)
  # start x at angle 0
  x = r
  y = 0
  glBegin(type)
  i = 0

  for i in range(num_segments):
    glVertex2f(x + cx, y + cy)
    tx = -y
    ty = x
    x = x + tx * tanf
    y = y + ty * tanf
    x = x * radf
    y = y * radf

  glEnd()

def glDrawCircle(cx,cy,radius,slices):

  glPushMatrix()
  glTranslatef(cx,cy,0.0)

# glBegin(GL_POLYGON)
# for v in xrange(0, slices):
#   angle  = float(v) * 2.0 * np.pi / slices
#   glVertex2f(np.cos(angle)*radius, np.sin(angle)*radius)
# glEnd()

  thickness = 1

  quadric = gluNewQuadric()
  gluDisk(quadric,radius-thickness,radius,slices,1)
  del quadric

  glPopMatrix()

def glDrawDisc(cx,cy,radius,slices):

  glPushMatrix()
  glTranslatef(cx,cy,0.0)

# glBegin(GL_POLYGON)
# for v in xrange(0, slices):
#   angle  = float(v) * 2.0 * np.pi / slices
#   glVertex2f(np.cos(angle)*radius, np.sin(angle)*radius)
# glEnd()

  quadric = gluNewQuadric()
  gluDisk(quadric,0,radius,slices,1)
  del quadric

  glPopMatrix()

def keyboard(bkey, x, y):
  global monitor, state
  global gazept
  global generate_video, video_out

  key = bkey.decode("utf-8")

  if key == '\033' or key == 'q':
    os._exit(0)

def mouse_motion(mx, my):

  global window1, window2

  win = glutGetWindow()

  if win == window1:

#   glutSetWindow(window1)

    ww = glutGet(GLUT_WINDOW_WIDTH)
    wh = glutGet(GLUT_WINDOW_HEIGHT)

    # get normalized coords
    x = float(mx)/ww
    y = float(my)/wh

    print("mouse motion (%f x %f)" % (float(x),float(y)))

  if win == window2:

#   glutSetWindow(window2)

    print("mouse motion in window2")

def main(argv):

  global window1, window2

  glutInit(argv)
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_ALPHA | GLUT_DEPTH)

  # get screen, window dimensions and position
  sw = glutGet(GLUT_SCREEN_WIDTH)
  sh = glutGet(GLUT_SCREEN_HEIGHT)
  # get desired window dimensions
  ww = 840
  wh = 525
  # figure out desired window position
  wx = (sw - ww)//2
  wy = (sh - wh)//2
  if sys.platform == 'win32':
    # subtract width of window decorations (Windows)
    wx -= 8
    wy -= 30

  print("screen res = ", sw, ' x ', sh)
  print("window res = ", ww, ' x ', wh)
  print("window pos = ", wx, ' x ', wy)

  window1 = None
  window2 = None

  size = (ww, wh)

  window1 = glutCreateWindow('Gazepoint client demo: press Esc to exit')
  glutReshapeWindow(ww,wh)
  glutPositionWindow(wx,wy)

  # set up callbacks
  glutReshapeFunc(reshape)
  glutDisplayFunc(display)
  glutIdleFunc(idle)
  glutKeyboardFunc(keyboard)
  glutPassiveMotionFunc(mouse_motion)
  glutTimerFunc(60,timer,0)

  window2 = glutCreateWindow('TM')
  glutReshapeWindow(500,500)
  glutPositionWindow(20,20)
  glutDisplayFunc(display)
  glutKeyboardFunc(keyboard)
  glutPassiveMotionFunc(mouse_motion)

  # make window current
  glutSetWindow(window1)

  # GL initialization
  glEnable(GL_LINE_SMOOTH)
  glShadeModel(GL_FLAT)

  # blending function
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
  glEnable(GL_BLEND)

# glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)
  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)

  glutMainLoop()

#################################
if __name__ == '__main__':
  main(sys.argv)
